@@ -18,7 +18,7 @@ if [ `uname` == "Darwin" ]; then
 	echo $'#include <stdio.h>\nint main() {\n  printf("Hello world\\n");\n}' > test.c
 	# Find out which architectures are supported by doing a test compile AND link per arch
   arch_flags=''
-	for arch in i386 x86_64 ppc ppc64; do
+	for arch in i386 x86_64 arm64; do
 		arch_flag=' -arch '$arch
 		if ${CC-cc} $arch_flag test.c 2> /dev/null; then
 			arch_flags+=$arch_flag
@@ -45,10 +45,11 @@ if [ `uname` == "Darwin" ]; then
 fi
 
 rm -rf readline-lib
-tar xzvf readline-8.0.tar.gz
-mv readline-8.0 readline-lib
+tar xzvf readline-8.1.tar.gz
+mv readline-8.1 readline-lib
 cd readline-lib
-# Future reminder: patch -p0 < ../readline80-001
+patch -p0 < ../readline81-001
+patch -p0 < ../readline81-002
 # Force compiler to CC/cc in the case of Darwin
 ./configure CPPFLAGS='-DNEED_EXTERN_PC -fPIC' $cc_override
 # Only the static libraries are required
